#include "NEC.h"
#include "stm32f4xx.h"


static uint8_t      bitn = 0;               // Bit number/repeat code number
static uint8_t      decoding_state = 0;     // What is decoding now? (adress, command, rep code)
static uint8_t      system = 0;             // System 8-bit adress
static uint8_t      inv_system = 0;         // Inverted 8-bit system
static uint8_t      command = 0;            // 8-bit command
static uint8_t      inv_command = 0;        // Inverted 8-bit command
static uint8_t      dec_ready = 0;          // Decoding is ready flag
static uint8_t      rep_code = 0;           // Quantity of repeat code sequencies

volatile uint8_t decoded_data=0;


static void GPIO_init()
	{

	RCC->AHB1ENR|=RCC_AHB1ENR_GPIOAEN;
	RCC->APB2ENR |=RCC_APB2ENR_SYSCFGEN;

	GPIOA->MODER&=~GPIO_MODER_MODE1;
	GPIOA->PUPDR|=GPIO_PUPDR_PUPD1_0;

	SYSCFG->EXTICR[0]|=SYSCFG_EXTICR1_EXTI1_PA;
	EXTI->IMR|=EXTI_IMR_IM1;
	EXTI->RTSR |=EXTI_RTSR_TR1;
	EXTI->FTSR |=EXTI_FTSR_TR1;
	NVIC_EnableIRQ(EXTI1_IRQn);

	}

static void TIM2_init()
	{
		RCC->APB1ENR|=RCC_APB1ENR_TIM2EN;
		TIM2->PSC=PRESCALER_10_US;
		TIM2->ARR=0xFFFF;
		TIM2->DIER=TIM_DIER_CC1IE;
		TIM2->CCR1=TIME_120_MS;
		TIM2->CR1 |= TIM_CR1_ARPE;
		TIM2->EGR|=TIM_EGR_CC1G;
		TIM2->CCER |= TIM_CCER_CC1E;
		TIM2->SR =0;
		NVIC_EnableIRQ(TIM2_IRQn);

	}





void NEC_Decoder_Init(void)
	{
	__disable_irq();
	GPIO_init();
	TIM2_init();
	__enable_irq();

	}






void reset_nec_timer(void)
{
	TIM2->CNT=0;
	TIM2->EGR|=TIM_EGR_UG;
}

uint16_t read_nec_timer(void)
{
  return TIM2->CNT; // Get the pulse len
}

void start_nec_timer(void)
{
  TIM2->CR1|=TIM_CR1_CEN;
  TIM2->CNT=0;
  TIM2->EGR|=TIM_EGR_UG;
}



void stop_nec_timer(void)
{
	TIM2->CR1&=~TIM_CR1_CEN;
}



void one_is_received(void)
{
  if (bitn < START_OF_COM)
  {
    if (bitn < START_OF_INV_SYS)     // Receive system address bits
    {
      system = ((system >> 1) | 0x80);
    }
    else
    {
      inv_system = ((inv_system >> 1) | 0x80);
    }
  }
  else
  {
    if (bitn < START_OF_INV_COM)     // Receive command bits
    {
      command = ((command >> 1) | 0x80);
    }
    else
    {
      inv_command = ((inv_command >> 1) | 0x80);
    }
  }
}


/* This functon stores the received "zero" bits
 *
 */
void zero_is_received(void)
{
  if (bitn < START_OF_COM)
  {
    if (bitn < START_OF_INV_SYS)   // Receive system address bits
    {
      system = ((system >> 1) & 0x7F);
    }
    else
    {
      inv_system = ((inv_system >> 1) & 0x7F);
    }
  }
  else
  {
    if (bitn < START_OF_INV_COM)   // Receive command bits
    {
      command = ((command >> 1) & 0x7F);
    }
    else
    {
      inv_command = ((inv_command >> 1) & 0x7F);
    }
  }
}



uint8_t decode_state()
{

	if((bitn==DECODE_IS_OK)){return decode_success;}
	else {return decode_failed;}
	return 0;
}

uint8_t get_system_state()
{
  return system;
}


uint8_t read_command()
{
	return decoded_data;

}

void TIM2_IRQHandler(void)
{

	if(TIM2->SR & TIM_SR_CC1IF)
	{ /*Interrupt from CC1*/
		TIM2->SR =0; /*Clear pending interrupt*/
		stop_nec_timer();
		/*Get the decoded data*/
		decoded_data=command;
		// clear NEC decoder variables
		bitn = 0;
		decoding_state = 0;
		system = 0;
		command = 0;
		inv_system = 0;
		inv_command = 0;
		dec_ready = 0;
		rep_code = 0;

	}

}


void EXTI1_IRQHandler(void)
	{
	EXTI->PR |=EXTI_PR_PR1; /*Clear pending interrupt*/
	uint16_t len = 0;

	  len = read_nec_timer();                       // Get the pulse len

	  switch (decoding_state)
	  {
	  case AGC_START:
	    start_nec_timer();                          // Start 9 ms pulse measurement
	    decoding_state = AGC_END;                   // Begin decoding
	    return;

	  case AGC_END:
	    reset_nec_timer();                          // Reset timer
	    if ((len > MIN_9_MS) && (len < MAX_9_MS))   // ... 9 ms AGC
	    {
	      decoding_state = SHORT_PAUSE;             // then begin a 4,5 ms..
	      return;                                   // ..pulse measurement
	    }
	    break;

	  case SHORT_PAUSE:
	    reset_nec_timer();                          // Reset timer
	    if ((len > MIN_4_5_MS) && (len < MAX_4_5_MS))
	    {
	      decoding_state = NEC_ADDR_COMM;           // Begin system adress decode
	      return;
	    }
	    break;

	  case NEC_ADDR_COMM:
	    if ((len > MIN_560_US) && (len < MAX_560_US) &&
	       (bitn == MAX_BITS) &&
	       ((system & inv_system) == 0) &&
	       ((command & inv_command) == 0))
	    {
	      decoding_state = LONG_PAUSE_1;            // Begin a long pause measurement
	      reset_nec_timer();                        // Reset timer
	      bitn = DECODE_IS_OK;
	      return;
	    }
	    else
	    {
	      decoding_state = NEC_BITS_DEC;
	      return;
	    }
	    break;

	  case NEC_BITS_DEC:                            // Bits receiving
	    reset_nec_timer();                          // Reset timer
	    decoding_state = NEC_ADDR_COMM;
	    bitn++;

	    if ((len > MIN_2_25_MS) && (len < MAX_2_25_MS))
	    {
	      one_is_received();                        // Received bit is "1"
	      return;
	    }
	    else if ((len > MIN_1_125_MS) && (len < MAX_1_125_MS))
	    {
	      zero_is_received();                       // Received bit is "0"
	      return;
	    }
	    break;

	  case LONG_PAUSE_1:
	    reset_nec_timer();                          // Reset timer
	    if ((len > MIN_40_MS) && (len < MAX_40_MS)) // P
	    {                                           // A
	      decoding_state = REPEAT_AGC;              // U
	      return;                                   // S
	    }                                           // E
	    break;

	  case REPEAT_AGC:
	    reset_nec_timer();                          // Reset timer
	    if ((len > MIN_9_MS) && (len < MAX_9_MS))
	    {                                           // R
	      decoding_state = REPEAT_SPACE;            // E
	      return;                                   // P
	    }                                           // E
	    break;                                      // A
	                                                // T
	  case REPEAT_SPACE:
	    reset_nec_timer();                          // Reset timer
	    if ((len > MIN_2_25_MS) && (len < MAX_2_25_MS))
	    {                                           // C
	      decoding_state = REPEAT_BURST;            // O
	      return;                                   // D
	    }                                           // E
	    break;

	  case REPEAT_BURST:
	    reset_nec_timer();                          // Reset timer
	    if ((len > MIN_560_US) && (len < MAX_560_US))
	    {                                           // R
	      decoding_state = LONG_PAUSE_2;            // E
	      rep_code++;                               // P
	      return;                                   // E
	    }                                           // A
	    break;                                      // T
	                                                // .
	  case LONG_PAUSE_2:                            // .
	    reset_nec_timer();                          // Reset timer
	    if ((len > MIN_98_MS) && (len < MAX_98_MS))
	    {                                           // C
	      decoding_state = REPEAT_AGC;              // O
	      return;                                   // D
	    }                                           // E
	    break;

	  default:
	    reset_nec_timer();                          // Reset timer
	    decoding_state = FAULT;                     // Indicate an error
	    dec_ready = 0;
	    break;
	  }

	}
